/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of JMS SMTP Bridge.
 * 
 * JMS SMTP Bridge is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * JMS SMTP Bridge is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with JMS SMTP Bridge. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.jmsmtpbridge.jms;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.StringTokenizer;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import org.apache.log4j.Logger;

import com.gt.jmsmtpbridge.smtp.SmtpSender;
import com.gt.jmsmtpbridge.smtp.SmtpSenderHome;
import com.gt.jmsmtpbridge.transform.Jms2SmtpTransformer;
import com.gt.jmsmtpbridge.transform.Jms2SmtpTransformerHome;

/**
 * The Message Driven Bean class responsible for listening for JMS messages that should be 
 * read and an email (or emails) must be sent based on contents of the message.
 * 
 * This Bean calls the Jms2SmtpTransformerBean to transform the JMS Message into a hashtable.
 * The hashtable is basically a hash of name/value pairs. It then creates parameters for sending 
 * the mail(s) from this hashtable. These parameters include the to, cc, bcc, subject and other 
 * email related fields. After that it calls the SmtpSenderBean to send mail(s).
 * 
 * @author The Shoe Shine Boy
 *
 */
@SuppressWarnings("serial")
public class JmsMessageReceiver implements MessageDrivenBean, MessageListener {
	// The logger
	private Logger m_logger = Logger.getLogger(getClass().getName());

	@Override
	public void ejbRemove() throws EJBException { }

	@Override
	public void setMessageDrivenContext(MessageDrivenContext ctx)
			throws EJBException { }
	
	public void ejbCreate() { }

	/**
	 * The onMessage method, which is called by the MDB container whenever this bean needs to 
	 * process a message.
	 * 
	 */
	@Override
	public void onMessage(Message msg) {
		try {
			m_logger.info("Inside onMessage().");
			InitialContext ctx = new InitialContext();
	
			m_logger.debug("Looking up JmsSmtpBridge-SmtpSender EJB...");
		  	Object smtpObj = ctx.lookup("JmsSmtpBridge-SmtpSender");
		  	m_logger.debug("EJB found. Creating handle...");
		  	SmtpSenderHome smtpHome = (SmtpSenderHome)PortableRemoteObject.
		  	narrow(smtpObj, SmtpSenderHome.class );  
		  	SmtpSender smtpEjb = smtpHome.create();
		  	m_logger.debug("Handle created.");
		  	
		  	m_logger.debug("Looking up JmsSmtpBridge-Jms2SmtpTransform EJB...");
		  	Object transformObj = ctx.lookup("JmsSmtpBridge-Jms2SmtpTransform");
		  	m_logger.debug("EJB found. Creating handle...");
		  	Jms2SmtpTransformerHome transformHome = (Jms2SmtpTransformerHome)PortableRemoteObject.
		  	narrow(transformObj, Jms2SmtpTransformerHome.class );  
		  	Jms2SmtpTransformer transformEjb = transformHome.create();
		  	m_logger.debug("Handle created.");
		  	
		  	m_logger.info("Calling JmsSmtpBridge-Jms2SmtpTransform EJB's transform() method...");
		  	Hashtable<String, Object> transformRes = transformEjb.transform(msg);

		  	//
		  	// The to, cc and bcc fields are sent as a comma separated string
		  	//
		  	
		  	Object obj = transformRes.get(Jms2SmtpTransformer.MAIL_TO);
		  	ArrayList<String> arr = new ArrayList<String>();
		  	String[] to = null;
		  	if (obj != null) {
		  		m_logger.debug("Found to: " + obj);
		  		StringTokenizer stTok = new StringTokenizer((String)obj, ",");
		  		while (stTok.hasMoreTokens()) {
		  			arr.add(stTok.nextToken());
		  		}
			  	to = new String[arr.size()];
			  	System.arraycopy(arr.toArray(), 0, to, 0, arr.size());
		  	} else {
		  		// move on
		  	}

		  	obj = transformRes.get(Jms2SmtpTransformer.MAIL_CC);
		  	arr = new ArrayList<String>();
		  	String[] cc = null;
		  	if (obj != null) {
		  		m_logger.debug("Found cc: " + obj);
		  		StringTokenizer stTok = new StringTokenizer((String)obj, ",");
		  		while (stTok.hasMoreTokens()) {
		  			arr.add(stTok.nextToken());
		  		}
			  	cc = new String[arr.size()];
			  	System.arraycopy(arr.toArray(), 0, cc, 0, arr.size());
		  	} else {
		  		// move on
		  	}
		  	
		  	obj = transformRes.get(Jms2SmtpTransformer.MAIL_BCC);
		  	arr = new ArrayList<String>();
		  	String[] bcc = null;
		  	if (obj != null) {
		  		m_logger.debug("Found bcc: " + obj);
		  		StringTokenizer stTok = new StringTokenizer((String)obj, ",");
		  		while (stTok.hasMoreTokens()) {
		  			arr.add(stTok.nextToken());
		  		}
			  	bcc = new String[arr.size()];
			  	System.arraycopy(arr.toArray(), 0, bcc, 0, arr.size());
		  	} else {
		  		// move on
		  	}
		  	
		  	obj = transformRes.get(Jms2SmtpTransformer.MAIL_SUBJECT);
		  	String subject = obj==null ? null : (String)obj;
		  	m_logger.debug("Found subject: " + subject);
		  	
		  	obj = transformRes.get(Jms2SmtpTransformer.MAIL_BODY);
		  	String body = obj==null ? null : (String)obj;
		  	m_logger.debug(body!=null ? "Found body length: " + body.length() : "Found body: null");
		  	
		  	obj = transformRes.get(Jms2SmtpTransformer.MAIL_SENDASHTML);
		  	boolean sendAsHtml = obj==null ? false : ((Boolean)obj).booleanValue();
		  	m_logger.debug("Found sendAsHtml Flag: " + (obj==null ? "null; defaulting to false" : sendAsHtml));

		  	m_logger.info("Calling JmsSmtpBridge-SmtpSender EJB's sendMail() method...");
		  	smtpEjb.sendMail(to, cc, bcc, subject, body, sendAsHtml);
		} catch (Exception e) {
			m_logger.error(e.getMessage());
			e.printStackTrace();
		}
	}

}
