/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of JMS SMTP Bridge.
 * 
 * JMS SMTP Bridge is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * JMS SMTP Bridge is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with JMS SMTP Bridge. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.test;

import java.util.Properties;
import java.util.StringTokenizer;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

/**
 * A sample stand-alone class to send a TextMessage so that it will be picked up by the JmsMessageReceiver 
 * MDB and eventually a mail would be sent out. The text in this message should be an xml of type:
 * 		<msg>
 * 			<to>customer1@abc.com,customer2@def.com</to>
 * 			<cc>customer3@abc.com,customer4@adf.com</cc>
 * 			<bcc>customer5@aa.com,customer6@re.com</bcc>
 * 			<subject>test subject</subject>
 * 			<body><![CDATA[test message]]></body>
 * 			<sendashtml>false</sendashtml>
 * 		</msg>
 * 
 * This class assumes that the MDB is listening on a queue named 'queue/JmsSmtpBridge-JmsReceiver' which 
 * resides in a local JBoss server. 
 * The important configuration parameters henceforth are:
 * Context.INITIAL_CONTEXT_FACTORY	org.jnp.interfaces.NamingContextFactory
 * Context.URL_PKG_PREFIXES			org.jboss.naming:org.jnp.interfaces
 * Context.PROVIDER_URL				jnp://localhost:1099
 *  
 * @author The Shoe Shine Boy
 *
 */
public class XmlMsgSender {

	public static void main(String[] args) throws Exception {
		System.out.println("Start");
		Properties objProperties = new Properties();
		objProperties.put(Context.INITIAL_CONTEXT_FACTORY,"org.jnp.interfaces.NamingContextFactory");
		objProperties.put(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
		objProperties.put(Context.PROVIDER_URL,"jnp://localhost:1099");
		InitialContext ctx = new InitialContext(objProperties);

		System.out.println("Looking up for the queue...");
		Queue q = (Queue)ctx.lookup("queue/JmsSmtpBridge-JmsReceiver");
		
		ConnectionFactory cf = (ConnectionFactory)ctx.lookup("QueueConnectionFactory");
		Connection c = cf.createConnection();
		Session s = c.createSession(false, Session.AUTO_ACKNOWLEDGE);
		
		MessageProducer sender = s.createProducer(q);
		
		TextMessage m = s.createTextMessage();
		m.setText(createTextMessage(args));

		System.out.println("Sending message...");
		sender.send(m);
		System.out.println("End");
	}
	
	private static String createTextMessage(String[] args) {
		String msg = "<msg>";
		
		for (int i=0; i<args.length; i++) {
			if (args[i].startsWith("to=")) {
				StringTokenizer stTok = new StringTokenizer(args[i].substring(3), ",");
				while (stTok.hasMoreTokens()) {
					msg = msg + "<to>" + stTok.nextToken() + "</to>";
				}
			} else if (args[i].startsWith("cc=")) {
				StringTokenizer stTok = new StringTokenizer(args[i].substring(3), ",");
				while (stTok.hasMoreTokens()) {
					msg = msg + "<cc>" + stTok.nextToken() + "</cc>";
				}
			} else if (args[i].startsWith("bcc=")) {
				StringTokenizer stTok = new StringTokenizer(args[i].substring(4), ",");
				while (stTok.hasMoreTokens()) {
					msg = msg + "<bcc>" + stTok.nextToken() + "</bcc>";
				}
			} else if (args[i].startsWith("subject=")) {
				msg = msg + "<subject>" + args[i].substring(8) + "</subject>";
			} else if (args[i].startsWith("body=")) {
				msg = msg + "<body><![CDATA[" + args[i].substring(5) + "]]></body>";
			} else if (args[i].startsWith("sendashtml=")) {
				msg = msg + "<sendashtml>" + args[i].substring(11) + "</sendashtml>";
			} else {
				// skip this argument
			}
		}
		
		msg += "</msg>";
		
		return msg;
	}
}
